"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createFromJson = exports.createFromIdls = exports.createFromRoot = void 0;
const nodes_1 = require("./nodes");
const visitors_1 = require("./visitors");
function createFromRoot(root, useDefaultVisitor = true) {
    let currentRoot = root;
    if (useDefaultVisitor) {
        currentRoot = (0, visitors_1.visit)(currentRoot, (0, visitors_1.defaultVisitor)());
    }
    return {
        getRoot() {
            return currentRoot;
        },
        getJson() {
            return JSON.stringify(currentRoot);
        },
        accept(visitor) {
            return (0, visitors_1.visit)(currentRoot, visitor);
        },
        update(visitor) {
            const newRoot = (0, visitors_1.visit)(currentRoot, visitor);
            (0, nodes_1.assertIsNode)(newRoot, 'rootNode');
            currentRoot = newRoot;
        },
        clone() {
            return createFromRoot((0, nodes_1.rootNode)(currentRoot.programs));
        },
    };
}
exports.createFromRoot = createFromRoot;
function createFromIdls(idls, useDefaultVisitor = true) {
    return createFromRoot((0, nodes_1.rootNodeFromIdls)(idls), useDefaultVisitor);
}
exports.createFromIdls = createFromIdls;
function createFromJson(json, useDefaultVisitor = false) {
    return createFromRoot(JSON.parse(json), useDefaultVisitor);
}
exports.createFromJson = createFromJson;
//# sourceMappingURL=Kinobi.js.map