"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getAllInstructionsWithSubs = exports.getAllInstructionArguments = exports.instructionNodeFromIdl = exports.instructionNode = void 0;
const shared_1 = require("../shared");
const InstructionAccountNode_1 = require("./InstructionAccountNode");
const InstructionArgumentNode_1 = require("./InstructionArgumentNode");
const Node_1 = require("./Node");
const discriminatorNodes_1 = require("./discriminatorNodes");
const TypeNode_1 = require("./typeNodes/TypeNode");
const valueNodes_1 = require("./valueNodes");
function instructionNode(input) {
    var _a, _b, _c, _d, _e;
    if (!input.name) {
        throw new shared_1.InvalidKinobiTreeError('InstructionNode must have a name.');
    }
    const name = (0, shared_1.mainCase)(input.name);
    return {
        kind: 'instructionNode',
        // Children.
        accounts: (_a = input.accounts) !== null && _a !== void 0 ? _a : [],
        arguments: (_b = input.arguments) !== null && _b !== void 0 ? _b : [],
        extraArguments: input.extraArguments,
        remainingAccounts: input.remainingAccounts,
        byteDeltas: input.byteDeltas,
        discriminators: input.discriminators,
        subInstructions: input.subInstructions,
        // Data.
        name,
        idlName: (_c = input.idlName) !== null && _c !== void 0 ? _c : input.name,
        docs: (_d = input.docs) !== null && _d !== void 0 ? _d : [],
        optionalAccountStrategy: (_e = input.optionalAccountStrategy) !== null && _e !== void 0 ? _e : 'programId',
    };
}
exports.instructionNode = instructionNode;
function instructionNodeFromIdl(idl) {
    var _a, _b, _c, _d;
    const idlName = (_a = idl.name) !== null && _a !== void 0 ? _a : '';
    const name = (0, shared_1.mainCase)(idlName);
    let dataArguments = ((_b = idl.args) !== null && _b !== void 0 ? _b : []).map(InstructionArgumentNode_1.instructionArgumentNodeFromIdl);
    let discriminators;
    if (idl.discriminant) {
        const discriminatorField = (0, InstructionArgumentNode_1.instructionArgumentNode)({
            name: 'discriminator',
            type: (0, TypeNode_1.createTypeNodeFromIdl)(idl.discriminant.type),
            defaultValue: (0, valueNodes_1.numberValueNode)(idl.discriminant.value),
            defaultValueStrategy: 'omitted',
        });
        dataArguments = [discriminatorField, ...dataArguments];
        discriminators = [(0, discriminatorNodes_1.fieldDiscriminatorNode)('discriminator')];
    }
    return instructionNode({
        name,
        idlName,
        docs: (_c = idl.docs) !== null && _c !== void 0 ? _c : [],
        accounts: ((_d = idl.accounts) !== null && _d !== void 0 ? _d : []).map((account) => (0, InstructionAccountNode_1.instructionAccountNodeFromIdl)(account)),
        arguments: dataArguments,
        discriminators,
        optionalAccountStrategy: idl.legacyOptionalAccountsStrategy
            ? 'omitted'
            : 'programId',
    });
}
exports.instructionNodeFromIdl = instructionNodeFromIdl;
function getAllInstructionArguments(node) {
    var _a;
    return [...node.arguments, ...((_a = node.extraArguments) !== null && _a !== void 0 ? _a : [])];
}
exports.getAllInstructionArguments = getAllInstructionArguments;
function getAllInstructionsWithSubs(node, config = {}) {
    const { leavesOnly = false, subInstructionsFirst = false } = config;
    if ((0, Node_1.isNode)(node, 'instructionNode')) {
        if (!node.subInstructions || node.subInstructions.length === 0)
            return [node];
        const subInstructions = node.subInstructions.flatMap((sub) => getAllInstructionsWithSubs(sub, config));
        if (leavesOnly)
            return subInstructions;
        return subInstructionsFirst
            ? [...subInstructions, node]
            : [node, ...subInstructions];
    }
    const instructions = (0, Node_1.isNode)(node, 'programNode')
        ? node.instructions
        : node.programs.flatMap((program) => program.instructions);
    return instructions.flatMap((instruction) => getAllInstructionsWithSubs(instruction, config));
}
exports.getAllInstructionsWithSubs = getAllInstructionsWithSubs;
//# sourceMappingURL=InstructionNode.js.map