"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.removeNullAndAssertIsNodeFilter = exports.assertIsNodeFilter = exports.isNodeFilter = exports.assertIsNode = exports.isNode = exports.REGISTERED_NODE_KINDS = void 0;
const utils_1 = require("../shared/utils");
const ContextualValueNode_1 = require("./contextualValueNodes/ContextualValueNode");
const DiscriminatorNode_1 = require("./discriminatorNodes/DiscriminatorNode");
const LinkNode_1 = require("./linkNodes/LinkNode");
const PdaSeedNode_1 = require("./pdaSeedNodes/PdaSeedNode");
const SizeNode_1 = require("./sizeNodes/SizeNode");
const TypeNode_1 = require("./typeNodes/TypeNode");
const ValueNode_1 = require("./valueNodes/ValueNode");
// Node Registration.
const REGISTERED_NODES = {
    rootNode: {},
    programNode: {},
    pdaNode: {},
    accountNode: {},
    instructionAccountNode: {},
    instructionArgumentNode: {},
    instructionByteDeltaNode: {},
    instructionNode: {},
    instructionRemainingAccountsNode: {},
    errorNode: {},
    definedTypeNode: {},
    // Groups.
    ...ContextualValueNode_1.REGISTERED_CONTEXTUAL_VALUE_NODES,
    ...DiscriminatorNode_1.REGISTERED_DISCRIMINATOR_NODES,
    ...LinkNode_1.REGISTERED_LINK_NODES,
    ...PdaSeedNode_1.REGISTERED_PDA_SEED_NODES,
    ...SizeNode_1.REGISTERED_SIZE_NODES,
    ...TypeNode_1.REGISTERED_TYPE_NODES,
    ...ValueNode_1.REGISTERED_VALUE_NODES,
};
exports.REGISTERED_NODE_KINDS = (0, utils_1.getNodeKinds)(REGISTERED_NODES);
// Node Helpers.
function isNode(node, kind) {
    const kinds = Array.isArray(kind) ? kind : [kind];
    return !!node && kinds.includes(node.kind);
}
exports.isNode = isNode;
function assertIsNode(node, kind) {
    var _a;
    const kinds = Array.isArray(kind) ? kind : [kind];
    if (!isNode(node, kinds)) {
        throw new Error(`Expected ${kinds.join(' | ')}, got ${(_a = node === null || node === void 0 ? void 0 : node.kind) !== null && _a !== void 0 ? _a : 'null'}.`);
    }
}
exports.assertIsNode = assertIsNode;
function isNodeFilter(kind) {
    return (node) => isNode(node, kind);
}
exports.isNodeFilter = isNodeFilter;
function assertIsNodeFilter(kind) {
    return (node) => {
        assertIsNode(node, kind);
        return true;
    };
}
exports.assertIsNodeFilter = assertIsNodeFilter;
function removeNullAndAssertIsNodeFilter(kind) {
    return (node) => {
        if (node)
            assertIsNode(node, kind);
        return node != null;
    };
}
exports.removeNullAndAssertIsNodeFilter = removeNullAndAssertIsNodeFilter;
//# sourceMappingURL=Node.js.map