"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.pdaNodeFromIdl = exports.pdaNode = void 0;
const shared_1 = require("../shared");
const pdaSeedNodes_1 = require("./pdaSeedNodes");
const sizeNodes_1 = require("./sizeNodes");
const typeNodes_1 = require("./typeNodes");
const valueNodes_1 = require("./valueNodes");
function pdaNode(name, seeds) {
    if (!name) {
        throw new shared_1.InvalidKinobiTreeError('PdaNode must have a name.');
    }
    return { kind: 'pdaNode', name: (0, shared_1.mainCase)(name), seeds };
}
exports.pdaNode = pdaNode;
function pdaNodeFromIdl(idl) {
    var _a, _b;
    const name = (0, shared_1.mainCase)((_a = idl.name) !== null && _a !== void 0 ? _a : '');
    const seeds = ((_b = idl.seeds) !== null && _b !== void 0 ? _b : []).map((seed) => {
        if (seed.kind === 'constant') {
            const type = (() => {
                if (seed.type === 'string')
                    return (0, typeNodes_1.stringTypeNode)({ size: (0, sizeNodes_1.remainderSizeNode)() });
                if (seed.type === 'bytes')
                    return (0, typeNodes_1.bytesTypeNode)((0, sizeNodes_1.remainderSizeNode)());
                return (0, typeNodes_1.createTypeNodeFromIdl)(seed.type);
            })();
            const value = (() => {
                if (typeof seed.value === 'string')
                    return (0, valueNodes_1.stringValueNode)(seed.value);
                if (typeof seed.value === 'number')
                    return (0, valueNodes_1.numberValueNode)(seed.value);
                return (0, valueNodes_1.booleanValueNode)(seed.value);
            })();
            return (0, pdaSeedNodes_1.constantPdaSeedNode)(type, value);
        }
        if (seed.kind === 'variable') {
            return (0, pdaSeedNodes_1.variablePdaSeedNode)(seed.name, (0, typeNodes_1.createTypeNodeFromIdl)(seed.type), seed.description ? [seed.description] : []);
        }
        return (0, pdaSeedNodes_1.programIdPdaSeedNode)();
    });
    return pdaNode(name, seeds);
}
exports.pdaNodeFromIdl = pdaNodeFromIdl;
//# sourceMappingURL=PdaNode.js.map