"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.isDataEnum = exports.isScalarEnum = exports.enumTypeNodeFromIdl = exports.enumTypeNode = void 0;
const EnumEmptyVariantTypeNode_1 = require("./EnumEmptyVariantTypeNode");
const EnumStructVariantTypeNode_1 = require("./EnumStructVariantTypeNode");
const EnumTupleVariantTypeNode_1 = require("./EnumTupleVariantTypeNode");
const NumberTypeNode_1 = require("./NumberTypeNode");
function enumTypeNode(variants, options = {}) {
    var _a;
    return {
        kind: 'enumTypeNode',
        variants,
        size: (_a = options.size) !== null && _a !== void 0 ? _a : (0, NumberTypeNode_1.numberTypeNode)('u8'),
    };
}
exports.enumTypeNode = enumTypeNode;
function enumTypeNodeFromIdl(idl) {
    const variants = idl.variants.map((variant) => {
        if (!variant.fields || variant.fields.length <= 0) {
            return (0, EnumEmptyVariantTypeNode_1.enumEmptyVariantTypeNodeFromIdl)(variant);
        }
        if (isStructField(variant.fields[0])) {
            return (0, EnumStructVariantTypeNode_1.enumStructVariantTypeNodeFromIdl)(variant);
        }
        return (0, EnumTupleVariantTypeNode_1.enumTupleVariantTypeNodeFromIdl)(variant);
    });
    return enumTypeNode(variants, {
        size: idl.size ? (0, NumberTypeNode_1.numberTypeNode)(idl.size) : undefined,
    });
}
exports.enumTypeNodeFromIdl = enumTypeNodeFromIdl;
function isScalarEnum(node) {
    return node.variants.every((variant) => variant.kind === 'enumEmptyVariantTypeNode');
}
exports.isScalarEnum = isScalarEnum;
function isDataEnum(node) {
    return !isScalarEnum(node);
}
exports.isDataEnum = isDataEnum;
function isStructField(field) {
    return typeof field === 'object' && 'name' in field && 'type' in field;
}
//# sourceMappingURL=EnumTypeNode.js.map