"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.isAsyncDefaultValue = exports.hasAsyncDefaultValues = exports.hasAsyncFunction = void 0;
const nodes_1 = require("../../nodes");
function hasAsyncFunction(instructionNode, resolvedInputs, asyncResolvers) {
    var _a, _b;
    const hasByteDeltasAsync = ((_a = instructionNode.byteDeltas) !== null && _a !== void 0 ? _a : []).some(({ value }) => (0, nodes_1.isNode)(value, 'resolverValueNode') && asyncResolvers.includes(value.name));
    const hasRemainingAccountsAsync = ((_b = instructionNode.remainingAccounts) !== null && _b !== void 0 ? _b : []).some(({ value }) => (0, nodes_1.isNode)(value, 'resolverValueNode') && asyncResolvers.includes(value.name));
    return (hasAsyncDefaultValues(resolvedInputs, asyncResolvers) ||
        hasByteDeltasAsync ||
        hasRemainingAccountsAsync);
}
exports.hasAsyncFunction = hasAsyncFunction;
function hasAsyncDefaultValues(resolvedInputs, asyncResolvers) {
    return resolvedInputs.some((input) => !!input.defaultValue &&
        isAsyncDefaultValue(input.defaultValue, asyncResolvers));
}
exports.hasAsyncDefaultValues = hasAsyncDefaultValues;
function isAsyncDefaultValue(defaultValue, asyncResolvers) {
    switch (defaultValue.kind) {
        case 'pdaValueNode':
            return true;
        case 'resolverValueNode':
            return asyncResolvers.includes(defaultValue.name);
        case 'conditionalValueNode':
            return (isAsyncDefaultValue(defaultValue.condition, asyncResolvers) ||
                (defaultValue.ifFalse == null
                    ? false
                    : isAsyncDefaultValue(defaultValue.ifFalse, asyncResolvers)) ||
                (defaultValue.ifTrue == null
                    ? false
                    : isAsyncDefaultValue(defaultValue.ifTrue, asyncResolvers)));
        default:
            return false;
    }
}
exports.isAsyncDefaultValue = isAsyncDefaultValue;
//# sourceMappingURL=asyncHelpers.js.map