"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getAccountPdaHelpersFragment = void 0;
const nodes_1 = require("../../../nodes");
const common_1 = require("./common");
function getAccountPdaHelpersFragment(scope) {
    const { accountNode, programNode, nameApi, linkables } = scope;
    const pdaNode = accountNode.pda ? linkables.get(accountNode.pda) : undefined;
    if (!pdaNode) {
        return (0, common_1.fragment)('');
    }
    const importFrom = 'generatedPdas';
    const pdaSeedsType = nameApi.pdaSeedsType(pdaNode.name);
    const findPdaFunction = nameApi.pdaFindFunction(pdaNode.name);
    const hasVariableSeeds = pdaNode.seeds.filter((0, nodes_1.isNodeFilter)('variablePdaSeedNode')).length > 0;
    return (0, common_1.fragmentFromTemplate)('accountPdaHelpers.njk', {
        accountType: nameApi.accountType(accountNode.name),
        accountMaybeType: nameApi.accountMaybeType(accountNode.name),
        pdaSeedsType,
        findPdaFunction,
        fetchFunction: nameApi.accountFetchFunction(accountNode.name),
        fetchMaybeFunction: nameApi.accountFetchMaybeFunction(accountNode.name),
        fetchFromSeedsFunction: nameApi.accountFetchFromSeedsFunction(accountNode.name),
        fetchMaybeFromSeedsFunction: nameApi.accountFetchMaybeFromSeedsFunction(accountNode.name),
        program: programNode,
        hasVariableSeeds,
    })
        .addImports(importFrom, hasVariableSeeds ? [pdaSeedsType, findPdaFunction] : [findPdaFunction])
        .addImports('solanaAddresses', ['Address'])
        .addImports('solanaAccounts', [
        'assertAccountExists',
        'FetchAccountConfig',
    ]);
}
exports.getAccountPdaHelpersFragment = getAccountPdaHelpersFragment;
//# sourceMappingURL=accountPdaHelpers.js.map