"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Fragment = exports.mergeFragments = exports.fragmentFromTemplate = exports.fragment = void 0;
const shared_1 = require("../../../shared");
const ImportMap_1 = require("../ImportMap");
function fragment(render, imports) {
    return new Fragment(render, imports);
}
exports.fragment = fragment;
function fragmentFromTemplate(fragmentFile, context, options) {
    return fragment((0, shared_1.resolveTemplate)(`${__dirname}/..`, `fragments/${fragmentFile}`, context, options));
}
exports.fragmentFromTemplate = fragmentFromTemplate;
function mergeFragments(fragments, mergeRenders) {
    return new Fragment(mergeRenders(fragments.map((f) => f.render)), new ImportMap_1.ImportMap().mergeWith(...fragments), new Set(fragments.flatMap((f) => [...f.features])));
}
exports.mergeFragments = mergeFragments;
class Fragment {
    constructor(render, imports, features) {
        this.render = render;
        this.imports = imports !== null && imports !== void 0 ? imports : new ImportMap_1.ImportMap();
        this.features = features !== null && features !== void 0 ? features : new Set();
    }
    setRender(render) {
        this.render = render;
        return this;
    }
    mapRender(fn) {
        this.render = fn(this.render);
        return this;
    }
    addImports(module, imports) {
        this.imports.add(module, imports);
        return this;
    }
    removeImports(module, imports) {
        this.imports.remove(module, imports);
        return this;
    }
    mergeImportsWith(...others) {
        this.imports.mergeWith(...others);
        return this;
    }
    addImportAlias(module, name, alias) {
        this.imports.addAlias(module, name, alias);
        return this;
    }
    addFeatures(features) {
        const featureArray = typeof features === 'string' ? [features] : features;
        featureArray.forEach((f) => this.features.add(f));
        return this;
    }
    removeFeatures(features) {
        const featureArray = typeof features === 'string' ? [features] : features;
        featureArray.forEach((f) => this.features.delete(f));
        return this;
    }
    hasFeatures(features) {
        const featureArray = typeof features === 'string' ? [features] : features;
        return featureArray.every((f) => this.features.has(f));
    }
    mergeFeaturesWith(...others) {
        others.forEach((f) => this.addFeatures([...f.features]));
        return this;
    }
    clone() {
        return new Fragment(this.render).mergeImportsWith(this.imports);
    }
    toString() {
        return this.render;
    }
}
exports.Fragment = Fragment;
//# sourceMappingURL=common.js.map