"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getInstructionByteDeltaFragment = void 0;
const nodes_1 = require("../../../nodes");
const shared_1 = require("../../../shared");
const common_1 = require("./common");
function getInstructionByteDeltaFragment(scope) {
    const { byteDeltas } = scope.instructionNode;
    const fragments = (byteDeltas !== null && byteDeltas !== void 0 ? byteDeltas : []).flatMap((r) => getByteDeltaFragment(r, scope));
    if (fragments.length === 0)
        return (0, common_1.fragment)('');
    return (0, common_1.mergeFragments)(fragments, (r) => `// Bytes created or reallocated by the instruction.\n` +
        `const byteDelta: number = [${r.join(',')}].reduce((a, b) => a + b, 0);`);
}
exports.getInstructionByteDeltaFragment = getInstructionByteDeltaFragment;
function getByteDeltaFragment(byteDelta, scope) {
    const bytesFragment = (() => {
        if ((0, nodes_1.isNode)(byteDelta.value, 'numberValueNode')) {
            return getNumberValueNodeFragment(byteDelta);
        }
        if ((0, nodes_1.isNode)(byteDelta.value, 'argumentValueNode')) {
            return getArgumentValueNodeFragment(byteDelta);
        }
        if ((0, nodes_1.isNode)(byteDelta.value, 'accountLinkNode')) {
            return getAccountLinkNodeFragment(byteDelta, scope);
        }
        if ((0, nodes_1.isNode)(byteDelta.value, 'resolverValueNode')) {
            return getResolverValueNodeFragment(byteDelta, scope);
        }
        return null;
    })();
    if (bytesFragment === null)
        return [];
    if (byteDelta.withHeader) {
        bytesFragment
            .mapRender((r) => `${r} + BASE_ACCOUNT_SIZE`)
            .addImports('solanaAccounts', 'BASE_ACCOUNT_SIZE');
    }
    if (byteDelta.subtract) {
        bytesFragment.mapRender((r) => `- (${r})`);
    }
    return [bytesFragment];
}
function getNumberValueNodeFragment(byteDelta) {
    (0, nodes_1.assertIsNode)(byteDelta.value, 'numberValueNode');
    return (0, common_1.fragment)(byteDelta.value.number.toString());
}
function getArgumentValueNodeFragment(byteDelta) {
    (0, nodes_1.assertIsNode)(byteDelta.value, 'argumentValueNode');
    const argumentName = (0, shared_1.camelCase)(byteDelta.value.name);
    return (0, common_1.fragment)(`Number(args.${argumentName})`);
}
function getAccountLinkNodeFragment(byteDelta, scope) {
    var _a;
    (0, nodes_1.assertIsNode)(byteDelta.value, 'accountLinkNode');
    const functionName = scope.nameApi.accountGetSizeFunction(byteDelta.value.name);
    const importFrom = (_a = byteDelta.value.importFrom) !== null && _a !== void 0 ? _a : 'generatedAccounts';
    return (0, common_1.fragment)(`${functionName}()`).addImports(importFrom, functionName);
}
function getResolverValueNodeFragment(byteDelta, scope) {
    var _a;
    (0, nodes_1.assertIsNode)(byteDelta.value, 'resolverValueNode');
    const isAsync = scope.asyncResolvers.includes(byteDelta.value.name);
    if (!scope.useAsync && isAsync)
        return null;
    const awaitKeyword = scope.useAsync && isAsync ? 'await ' : '';
    const functionName = scope.nameApi.resolverFunction(byteDelta.value.name);
    return (0, common_1.fragment)(`${awaitKeyword}${functionName}(resolverScope)`)
        .addImports((_a = byteDelta.value.importFrom) !== null && _a !== void 0 ? _a : 'hooked', functionName)
        .addFeatures(['instruction:resolverScopeVariable']);
}
//# sourceMappingURL=instructionByteDelta.js.map