"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getProgramAccountsFragment = void 0;
const common_1 = require("./common");
const discriminatorCondition_1 = require("./discriminatorCondition");
function getProgramAccountsFragment(scope) {
    if (scope.programNode.accounts.length === 0)
        return (0, common_1.fragment)('');
    return (0, common_1.mergeFragments)([
        getProgramAccountsEnumFragment(scope),
        getProgramAccountsIdentifierFunctionFragment(scope),
    ], (r) => `${r.join('\n\n')}\n`);
}
exports.getProgramAccountsFragment = getProgramAccountsFragment;
function getProgramAccountsEnumFragment(scope) {
    const { programNode, nameApi } = scope;
    const programAccountsEnum = nameApi.programAccountsEnum(programNode.name);
    const programAccountsEnumVariants = programNode.accounts.map((account) => nameApi.programAccountsEnumVariant(account.name));
    return (0, common_1.fragment)(`export enum ${programAccountsEnum} { ` +
        `${programAccountsEnumVariants.join(', ')}` +
        ` }`);
}
function getProgramAccountsIdentifierFunctionFragment(scope) {
    const { programNode, nameApi } = scope;
    const accountsWithDiscriminators = programNode.accounts.filter((account) => { var _a; return ((_a = account.discriminators) !== null && _a !== void 0 ? _a : []).length > 0; });
    const hasAccountDiscriminators = accountsWithDiscriminators.length > 0;
    if (!hasAccountDiscriminators)
        return (0, common_1.fragment)('');
    const programAccountsEnum = nameApi.programAccountsEnum(programNode.name);
    const programAccountsIdentifierFunction = nameApi.programAccountsIdentifierFunction(programNode.name);
    const discriminatorsFragment = (0, common_1.mergeFragments)(accountsWithDiscriminators.map((account) => {
        var _a;
        const variant = nameApi.programAccountsEnumVariant(account.name);
        return (0, discriminatorCondition_1.getDiscriminatorConditionFragment)({
            ...scope,
            discriminators: (_a = account.discriminators) !== null && _a !== void 0 ? _a : [],
            struct: account.data,
            dataName: 'data',
            ifTrue: `return ${programAccountsEnum}.${variant};`,
        });
    }), (r) => r.join('\n'));
    return discriminatorsFragment.mapRender((discriminators) => `export function ${programAccountsIdentifierFunction}(` +
        `account: { data: Uint8Array } | Uint8Array` +
        `): ${programAccountsEnum} {\n` +
        `const data = account instanceof Uint8Array ? account : account.data;\n` +
        `${discriminators}\n` +
        `throw new Error("The provided account could not be identified as a ${programNode.name} account.")\n` +
        `}`);
}
//# sourceMappingURL=programAccounts.js.map