"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.renderValueNodeVisitor = void 0;
const nodes_1 = require("../../nodes");
const visitors_1 = require("../../visitors");
const fragments_1 = require("./fragments");
function renderValueNodeVisitor(input) {
    const { nameApi, linkables, nonScalarEnums } = input;
    return {
        visitArrayValue(node) {
            return (0, fragments_1.mergeFragments)(node.items.map((v) => (0, visitors_1.visit)(v, this)), (renders) => `[${renders.join(', ')}]`);
        },
        visitBooleanValue(node) {
            return (0, fragments_1.fragment)(JSON.stringify(node.boolean));
        },
        visitEnumValue(node) {
            var _a, _b;
            const enumName = nameApi.dataType(node.enum.name);
            const enumFunction = nameApi.dataEnumFunction(node.enum.name);
            const importFrom = (_a = node.enum.importFrom) !== null && _a !== void 0 ? _a : 'generatedTypes';
            const enumNode = (_b = linkables.get(node.enum)) === null || _b === void 0 ? void 0 : _b.type;
            const isScalar = enumNode && (0, nodes_1.isNode)(enumNode, 'enumTypeNode')
                ? (0, nodes_1.isScalarEnum)(enumNode)
                : !nonScalarEnums.includes(node.enum.name);
            if (!node.value && isScalar) {
                const variantName = nameApi.scalarEnumVariant(node.variant);
                return (0, fragments_1.fragment)(`${enumName}.${variantName}`).addImports(importFrom, enumName);
            }
            const variantName = nameApi.dataEnumVariant(node.variant);
            if (!node.value) {
                return (0, fragments_1.fragment)(`${enumFunction}('${variantName}')`).addImports(importFrom, enumFunction);
            }
            return (0, visitors_1.visit)(node.value, this)
                .mapRender((r) => `${enumFunction}('${variantName}', ${r})`)
                .addImports(importFrom, enumFunction);
        },
        visitMapValue(node) {
            const entryFragments = node.entries.map((entry) => (0, visitors_1.visit)(entry, this));
            return (0, fragments_1.mergeFragments)(entryFragments, (renders) => `new Map([${renders.join(', ')}])`);
        },
        visitMapEntryValue(node) {
            return (0, fragments_1.mergeFragments)([(0, visitors_1.visit)(node.key, this), (0, visitors_1.visit)(node.value, this)], (renders) => `[${renders.join(', ')}]`);
        },
        visitNoneValue() {
            return (0, fragments_1.fragment)('none()').addImports('solanaOptions', 'none');
        },
        visitNumberValue(node) {
            return (0, fragments_1.fragment)(JSON.stringify(node.number));
        },
        visitPublicKeyValue(node) {
            return (0, fragments_1.fragment)(`address("${node.publicKey}")`).addImports('solanaAddresses', 'address');
        },
        visitSetValue(node) {
            return (0, fragments_1.mergeFragments)(node.items.map((v) => (0, visitors_1.visit)(v, this)), (renders) => `new Set([${renders.join(', ')}])`);
        },
        visitSomeValue(node) {
            return (0, visitors_1.visit)(node.value, this)
                .mapRender((r) => `some(${r})`)
                .addImports('solanaOptions', 'some');
        },
        visitStringValue(node) {
            return (0, fragments_1.fragment)(JSON.stringify(node.string));
        },
        visitStructValue(node) {
            return (0, fragments_1.mergeFragments)(node.fields.map((field) => (0, visitors_1.visit)(field, this)), (renders) => `{ ${renders.join(', ')} }`);
        },
        visitStructFieldValue(node) {
            return (0, visitors_1.visit)(node.value, this).mapRender((r) => `${node.name}: ${r}`);
        },
        visitTupleValue(node) {
            return (0, fragments_1.mergeFragments)(node.items.map((v) => (0, visitors_1.visit)(v, this)), (renders) => `[${renders.join(', ')}]`);
        },
    };
}
exports.renderValueNodeVisitor = renderValueNodeVisitor;
//# sourceMappingURL=renderValueNodeVisitor.js.map