"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.JavaScriptContextMap = void 0;
class JavaScriptContextMap {
    constructor() {
        this._interfaces = new Set();
    }
    add(contextInterface) {
        if (Array.isArray(contextInterface)) {
            contextInterface.forEach((i) => this._interfaces.add(i));
        }
        else {
            this._interfaces.add(contextInterface);
        }
        return this;
    }
    remove(contextInterface) {
        if (Array.isArray(contextInterface)) {
            contextInterface.forEach((i) => this._interfaces.delete(i));
        }
        else {
            this._interfaces.delete(contextInterface);
        }
        return this;
    }
    mergeWith(...others) {
        others.forEach((other) => this.add([...other._interfaces]));
        return this;
    }
    isEmpty() {
        return this._interfaces.size === 0;
    }
    toString() {
        const contextInterfaces = [...this._interfaces]
            .sort()
            .map((i) => `"${i}"`)
            .join(' | ');
        return `Pick<Context, ${contextInterfaces}>`;
    }
}
exports.JavaScriptContextMap = JavaScriptContextMap;
//# sourceMappingURL=JavaScriptContextMap.js.map