"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.renderRustVisitor = void 0;
const child_process_1 = require("child_process");
const shared_1 = require("../../shared");
const visitors_1 = require("../../visitors");
const getRenderMapVisitor_1 = require("./getRenderMapVisitor");
function renderRustVisitor(path, options = {}) {
    return (0, visitors_1.rootNodeVisitor)((root) => {
        var _a;
        // Delete existing generated folder.
        if ((_a = options.deleteFolderBeforeRendering) !== null && _a !== void 0 ? _a : true) {
            (0, shared_1.deleteFolder)(path);
        }
        // Render the new files.
        (0, visitors_1.visit)(root, (0, visitors_1.writeRenderMapVisitor)((0, getRenderMapVisitor_1.getRenderMapVisitor)(options), path));
        // format the code
        if (options.formatCode) {
            if (options.crateFolder) {
                runFormatter('cargo-fmt', [
                    '--manifest-path',
                    `${options.crateFolder}/Cargo.toml`,
                ]);
            }
            else {
                (0, shared_1.logWarn)('No crate folder specified, skipping formatting.');
            }
        }
    });
}
exports.renderRustVisitor = renderRustVisitor;
function runFormatter(cmd, args) {
    var _a;
    const { stdout, stderr, error } = (0, child_process_1.spawnSync)(cmd, args);
    if ((_a = error === null || error === void 0 ? void 0 : error.message) === null || _a === void 0 ? void 0 : _a.includes('ENOENT')) {
        (0, shared_1.logWarn)(`Could not find ${cmd}, skipping formatting.`);
        return;
    }
    if (stdout.length > 0) {
        (0, shared_1.logWarn)(`(cargo-fmt) ${stdout || error}`);
    }
    if (stderr.length > 0) {
        (0, shared_1.logError)(`(cargo-fmt) ${stderr || error}`);
    }
}
//# sourceMappingURL=renderRustVisitor.js.map