"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.RenderMap = void 0;
const errors_1 = require("./errors");
const utils_1 = require("./utils");
class RenderMap {
    constructor() {
        this._map = new Map();
    }
    add(relativePath, code) {
        this._map.set(relativePath, code);
        return this;
    }
    remove(relativePath) {
        this._map.delete(relativePath);
        return this;
    }
    mergeWith(...others) {
        others.forEach((other) => {
            other._map.forEach((code, relativePath) => {
                this.add(relativePath, code);
            });
        });
        return this;
    }
    isEmpty() {
        return this._map.size === 0;
    }
    has(key) {
        return this._map.has(key);
    }
    get(key) {
        const value = this.safeGet(key);
        if (value === undefined) {
            throw new errors_1.KinobiError(`Cannot find key "${key}" in RenderMap.`);
        }
        return value;
    }
    safeGet(key) {
        return this._map.get(key);
    }
    contains(key, value) {
        const content = this.get(key);
        return typeof value === 'string'
            ? content.includes(value)
            : value.test(content);
    }
    write(path) {
        this._map.forEach((code, relativePath) => {
            (0, utils_1.createFile)(`${path}/${relativePath}`, code);
        });
    }
}
exports.RenderMap = RenderMap;
//# sourceMappingURL=RenderMap.js.map