"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.deleteFolder = exports.resolveTemplate = exports.createFile = exports.createDirectory = exports.readJson = exports.rustDocblock = exports.jsDocblock = exports.mainCase = exports.snakeCase = exports.kebabCase = exports.camelCase = exports.pascalCase = exports.titleCase = exports.capitalize = exports.getNodeKinds = void 0;
const fs_1 = require("fs");
const nunjucks_1 = __importDefault(require("nunjucks"));
function getNodeKinds(obj) {
    return Object.keys(obj);
}
exports.getNodeKinds = getNodeKinds;
function capitalize(str) {
    if (str.length === 0)
        return str;
    return str.charAt(0).toUpperCase() + str.slice(1).toLowerCase();
}
exports.capitalize = capitalize;
function titleCase(str) {
    return str
        .replace(/([A-Z])/g, ' $1')
        .split(/[-_\s+.]/)
        .filter((word) => word.length > 0)
        .map(capitalize)
        .join(' ');
}
exports.titleCase = titleCase;
function pascalCase(str) {
    return titleCase(str).split(' ').join('');
}
exports.pascalCase = pascalCase;
function camelCase(str) {
    if (str.length === 0)
        return str;
    const pascalStr = pascalCase(str);
    return pascalStr.charAt(0).toLowerCase() + pascalStr.slice(1);
}
exports.camelCase = camelCase;
function kebabCase(str) {
    return titleCase(str).split(' ').join('-').toLowerCase();
}
exports.kebabCase = kebabCase;
function snakeCase(str) {
    return titleCase(str).split(' ').join('_').toLowerCase();
}
exports.snakeCase = snakeCase;
function mainCase(str) {
    return camelCase(str);
}
exports.mainCase = mainCase;
function jsDocblock(docs) {
    if (docs.length <= 0)
        return '';
    if (docs.length === 1)
        return `/** ${docs[0]} */\n`;
    const lines = docs.map((doc) => ` * ${doc}`);
    return `/**\n${lines.join('\n')}\n */\n`;
}
exports.jsDocblock = jsDocblock;
function rustDocblock(docs) {
    if (docs.length <= 0)
        return '';
    const lines = docs.map((doc) => `/// ${doc}`);
    return `${lines.join('\n')}\n`;
}
exports.rustDocblock = rustDocblock;
function readJson(value) {
    return JSON.parse((0, fs_1.readFileSync)(value, 'utf-8'));
}
exports.readJson = readJson;
const createDirectory = (path) => {
    (0, fs_1.mkdirSync)(path, { recursive: true });
};
exports.createDirectory = createDirectory;
const createFile = (path, content) => {
    const directory = path.substring(0, path.lastIndexOf('/'));
    if (!(0, fs_1.existsSync)(directory)) {
        (0, exports.createDirectory)(directory);
    }
    (0, fs_1.writeFileSync)(path, content);
};
exports.createFile = createFile;
const resolveTemplate = (directory, file, context, options) => {
    const env = nunjucks_1.default.configure(directory, {
        trimBlocks: true,
        autoescape: false,
        ...options,
    });
    env.addFilter('pascalCase', pascalCase);
    env.addFilter('camelCase', camelCase);
    env.addFilter('snakeCase', snakeCase);
    env.addFilter('kebabCase', kebabCase);
    env.addFilter('titleCase', titleCase);
    env.addFilter('jsDocblock', jsDocblock);
    env.addFilter('rustDocblock', rustDocblock);
    return env.render(file, context);
};
exports.resolveTemplate = resolveTemplate;
const deleteFolder = (path) => {
    if ((0, fs_1.existsSync)(path)) {
        (0, fs_1.rmSync)(path, { recursive: true });
    }
};
exports.deleteFolder = deleteFolder;
//# sourceMappingURL=utils.js.map