"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createSubInstructionsFromEnumArgsVisitor = void 0;
const nodes_1 = require("../nodes");
const shared_1 = require("../shared");
const bottomUpTransformerVisitor_1 = require("./bottomUpTransformerVisitor");
const flattenInstructionDataArgumentsVisitor_1 = require("./flattenInstructionDataArgumentsVisitor");
const recordLinkablesVisitor_1 = require("./recordLinkablesVisitor");
function createSubInstructionsFromEnumArgsVisitor(map) {
    const linkables = new shared_1.LinkableDictionary();
    const visitor = (0, bottomUpTransformerVisitor_1.bottomUpTransformerVisitor)(Object.entries(map).map(([selector, argNameInput]) => ({
        select: ['[instructionNode]', selector],
        transform: (node) => {
            var _a, _b, _c;
            (0, nodes_1.assertIsNode)(node, 'instructionNode');
            const argFields = node.arguments;
            const argName = (0, shared_1.mainCase)(argNameInput);
            const argFieldIndex = argFields.findIndex((field) => field.name === argName);
            const argField = argFieldIndex >= 0 ? argFields[argFieldIndex] : null;
            if (!argField) {
                (0, shared_1.logWarn)(`Could not find instruction argument [${argName}].`);
                return node;
            }
            let argType;
            if ((0, nodes_1.isNode)(argField.type, 'enumTypeNode')) {
                argType = argField.type;
            }
            else if ((0, nodes_1.isNode)(argField.type, 'definedTypeLinkNode') &&
                linkables.has(argField.type)) {
                const linkedType = (_b = (_a = linkables.get(argField.type)) === null || _a === void 0 ? void 0 : _a.type) !== null && _b !== void 0 ? _b : null;
                (0, nodes_1.assertIsNode)(linkedType, 'enumTypeNode');
                argType = linkedType;
            }
            else {
                (0, shared_1.logWarn)(`Could not find an enum type for ` +
                    `instruction argument [${argName}].`);
                return node;
            }
            const subInstructions = argType.variants.map((variant, index) => {
                const subName = (0, shared_1.mainCase)(`${node.name} ${variant.name}`);
                const subFields = argFields.slice(0, argFieldIndex);
                subFields.push((0, nodes_1.instructionArgumentNode)({
                    name: `${subName}Discriminator`,
                    type: (0, nodes_1.numberTypeNode)('u8'),
                    defaultValue: (0, nodes_1.numberValueNode)(index),
                    defaultValueStrategy: 'omitted',
                }));
                if ((0, nodes_1.isNode)(variant, 'enumStructVariantTypeNode')) {
                    subFields.push((0, nodes_1.instructionArgumentNode)({
                        ...argField,
                        type: variant.struct,
                    }));
                }
                else if ((0, nodes_1.isNode)(variant, 'enumTupleVariantTypeNode')) {
                    subFields.push((0, nodes_1.instructionArgumentNode)({
                        ...argField,
                        type: variant.tuple,
                    }));
                }
                subFields.push(...argFields.slice(argFieldIndex + 1));
                return (0, nodes_1.instructionNode)({
                    ...node,
                    name: subName,
                    arguments: (0, flattenInstructionDataArgumentsVisitor_1.flattenInstructionArguments)(subFields),
                });
            });
            return (0, nodes_1.instructionNode)({
                ...node,
                subInstructions: [
                    ...((_c = node.subInstructions) !== null && _c !== void 0 ? _c : []),
                    ...subInstructions,
                ],
            });
        },
    })));
    return (0, recordLinkablesVisitor_1.recordLinkablesVisitor)(visitor, linkables);
}
exports.createSubInstructionsFromEnumArgsVisitor = createSubInstructionsFromEnumArgsVisitor;
//# sourceMappingURL=createSubInstructionsFromEnumArgsVisitor.js.map