"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.extendVisitor = void 0;
const nodes_1 = require("../nodes");
const visitor_1 = require("./visitor");
function extendVisitor(visitor, overrides) {
    const registeredVisitFunctions = nodes_1.REGISTERED_NODE_KINDS.map(visitor_1.getVisitFunctionName);
    const overriddenFunctions = Object.fromEntries(Object.keys(overrides).flatMap((key) => {
        if (!registeredVisitFunctions.includes(key)) {
            return [];
        }
        const castedKey = key;
        if (!visitor[castedKey]) {
            throw new Error(`Cannot extend visitor with function "${castedKey}" as the base visitor does not support it.`);
        }
        return [
            [
                castedKey,
                function extendedVisitNode(node) {
                    const extendedFunction = overrides[castedKey];
                    const nextFunction = visitor[castedKey];
                    return extendedFunction.bind(this)(node, {
                        next: nextFunction.bind(this),
                        self: this,
                    });
                },
            ],
        ];
    }));
    return {
        ...visitor,
        ...overriddenFunctions,
    };
}
exports.extendVisitor = extendVisitor;
//# sourceMappingURL=extendVisitor.js.map