"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.interceptVisitor = void 0;
const nodes_1 = require("../nodes");
const visitor_1 = require("./visitor");
function interceptVisitor(visitor, interceptor) {
    const registeredVisitFunctions = nodes_1.REGISTERED_NODE_KINDS.map(visitor_1.getVisitFunctionName);
    return Object.fromEntries(Object.keys(visitor).flatMap((key) => {
        if (!registeredVisitFunctions.includes(key)) {
            return [];
        }
        const castedKey = key;
        return [
            [
                castedKey,
                function interceptedVisitNode(node) {
                    const baseFunction = visitor[castedKey];
                    return interceptor(node, baseFunction.bind(this));
                },
            ],
        ];
    }));
}
exports.interceptVisitor = interceptVisitor;
//# sourceMappingURL=interceptVisitor.js.map