"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.setAccountDiscriminatorFromFieldVisitor = void 0;
const nodes_1 = require("../nodes");
const bottomUpTransformerVisitor_1 = require("./bottomUpTransformerVisitor");
function setAccountDiscriminatorFromFieldVisitor(map) {
    return (0, bottomUpTransformerVisitor_1.bottomUpTransformerVisitor)(Object.entries(map).map(([selector, { field, value, offset },]) => ({
        select: ['[accountNode]', selector],
        transform: (node) => {
            var _a;
            (0, nodes_1.assertIsNode)(node, 'accountNode');
            const fieldIndex = node.data.fields.findIndex((f) => f.name === field);
            if (fieldIndex < 0) {
                throw new Error(`Account [${node.name}] does not have a field named [${field}].`);
            }
            const fieldNode = node.data.fields[fieldIndex];
            return (0, nodes_1.accountNode)({
                ...node,
                discriminators: [
                    (0, nodes_1.fieldDiscriminatorNode)(field, offset),
                    ...((_a = node.discriminators) !== null && _a !== void 0 ? _a : []),
                ],
                data: (0, nodes_1.structTypeNode)([
                    ...node.data.fields.slice(0, fieldIndex),
                    (0, nodes_1.structFieldTypeNode)({
                        ...fieldNode,
                        defaultValue: value,
                        defaultValueStrategy: 'omitted',
                    }),
                    ...node.data.fields.slice(fieldIndex + 1),
                ]),
            });
        },
    })));
}
exports.setAccountDiscriminatorFromFieldVisitor = setAccountDiscriminatorFromFieldVisitor;
//# sourceMappingURL=setAccountDiscriminatorFromFieldVisitor.js.map