"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.unwrapInstructionArgsDefinedTypesVisitor = void 0;
const nodes_1 = require("../nodes");
const getDefinedTypeHistogramVisitor_1 = require("./getDefinedTypeHistogramVisitor");
const singleNodeVisitor_1 = require("./singleNodeVisitor");
const unwrapDefinedTypesVisitor_1 = require("./unwrapDefinedTypesVisitor");
const visitor_1 = require("./visitor");
function unwrapInstructionArgsDefinedTypesVisitor() {
    return (0, singleNodeVisitor_1.rootNodeVisitor)((root) => {
        const histogram = (0, visitor_1.visit)(root, (0, getDefinedTypeHistogramVisitor_1.getDefinedTypeHistogramVisitor)());
        const allDefinedTypes = (0, nodes_1.getAllDefinedTypes)(root);
        const definedTypesToInline = Object.keys(histogram)
            // Get all defined types used exactly once as an instruction argument.
            .filter((name) => {
            var _a, _b;
            return ((_a = histogram[name].total) !== null && _a !== void 0 ? _a : 0) === 1 &&
                ((_b = histogram[name].directlyAsInstructionArgs) !== null && _b !== void 0 ? _b : 0) ===
                    1;
        })
            // Filter out enums which are better defined as external types.
            .filter((name) => {
            const found = allDefinedTypes.find((type) => type.name === name);
            return found && !(0, nodes_1.isNode)(found.type, 'enumTypeNode');
        });
        // Inline the identified defined types if any.
        if (definedTypesToInline.length > 0) {
            const inlineVisitor = (0, unwrapDefinedTypesVisitor_1.unwrapDefinedTypesVisitor)(definedTypesToInline);
            const newRoot = (0, visitor_1.visit)(root, inlineVisitor);
            (0, nodes_1.assertIsNode)(newRoot, 'rootNode');
            return newRoot;
        }
        return root;
    });
}
exports.unwrapInstructionArgsDefinedTypesVisitor = unwrapInstructionArgsDefinedTypesVisitor;
//# sourceMappingURL=unwrapInstructionArgsDefinedTypesVisitor.js.map