import type { IdlInstruction } from '../idl';
import { MainCaseString } from '../shared';
import { InstructionAccountNode } from './InstructionAccountNode';
import { InstructionArgumentNode } from './InstructionArgumentNode';
import { InstructionByteDeltaNode } from './InstructionByteDeltaNode';
import { InstructionRemainingAccountsNode } from './InstructionRemainingAccountsNode';
import { ProgramNode } from './ProgramNode';
import { RootNode } from './RootNode';
import { DiscriminatorNode } from './discriminatorNodes';
export type InstructionNode = {
    readonly kind: 'instructionNode';
    readonly accounts: InstructionAccountNode[];
    readonly arguments: InstructionArgumentNode[];
    readonly extraArguments?: InstructionArgumentNode[];
    readonly remainingAccounts?: InstructionRemainingAccountsNode[];
    readonly byteDeltas?: InstructionByteDeltaNode[];
    readonly discriminators?: DiscriminatorNode[];
    readonly subInstructions?: InstructionNode[];
    readonly name: MainCaseString;
    readonly idlName: string;
    readonly docs: string[];
    readonly optionalAccountStrategy: 'omitted' | 'programId';
};
export type InstructionNodeInput = Omit<Partial<InstructionNode>, 'kind' | 'name'> & {
    readonly name: string;
};
export declare function instructionNode(input: InstructionNodeInput): InstructionNode;
export declare function instructionNodeFromIdl(idl: Partial<IdlInstruction>): InstructionNode;
export declare function getAllInstructionArguments(node: InstructionNode): InstructionArgumentNode[];
export declare function getAllInstructionsWithSubs(node: ProgramNode | RootNode | InstructionNode, config?: {
    leavesOnly?: boolean;
    subInstructionsFirst?: boolean;
}): InstructionNode[];
