import type { AccountNode } from './AccountNode';
import type { DefinedTypeNode } from './DefinedTypeNode';
import type { ErrorNode } from './ErrorNode';
import type { InstructionAccountNode } from './InstructionAccountNode';
import type { InstructionArgumentNode } from './InstructionArgumentNode';
import type { InstructionByteDeltaNode } from './InstructionByteDeltaNode';
import type { InstructionNode } from './InstructionNode';
import type { InstructionRemainingAccountsNode } from './InstructionRemainingAccountsNode';
import type { PdaNode } from './PdaNode';
import type { ProgramNode } from './ProgramNode';
import type { RootNode } from './RootNode';
declare const REGISTERED_NODES: {
    mapEntryValueNode: import("./valueNodes").MapEntryValueNode;
    structFieldValueNode: import("./valueNodes").StructFieldValueNode;
    arrayValueNode: import("./valueNodes").ArrayValueNode;
    booleanValueNode: import("./valueNodes").BooleanValueNode;
    enumValueNode: import("./valueNodes").EnumValueNode;
    mapValueNode: import("./valueNodes").MapValueNode;
    noneValueNode: import("./valueNodes").NoneValueNode;
    numberValueNode: import("./valueNodes").NumberValueNode;
    setValueNode: import("./valueNodes").SetValueNode;
    someValueNode: import("./valueNodes").SomeValueNode;
    structValueNode: import("./valueNodes").StructValueNode;
    tupleValueNode: import("./valueNodes").TupleValueNode;
    publicKeyValueNode: import("./valueNodes").PublicKeyValueNode;
    stringValueNode: import("./valueNodes").StringValueNode;
    structFieldTypeNode: import("./typeNodes").StructFieldTypeNode;
    enumEmptyVariantTypeNode: import("./typeNodes").EnumEmptyVariantTypeNode;
    enumStructVariantTypeNode: import("./typeNodes").EnumStructVariantTypeNode;
    enumTupleVariantTypeNode: import("./typeNodes").EnumTupleVariantTypeNode;
    amountTypeNode: import("./typeNodes").AmountTypeNode;
    arrayTypeNode: import("./typeNodes").ArrayTypeNode;
    booleanTypeNode: import("./typeNodes").BooleanTypeNode;
    bytesTypeNode: import("./typeNodes").BytesTypeNode;
    dateTimeTypeNode: import("./typeNodes").DateTimeTypeNode;
    enumTypeNode: import("./typeNodes").EnumTypeNode;
    mapTypeNode: import("./typeNodes").MapTypeNode;
    numberTypeNode: import("./typeNodes").NumberTypeNode;
    optionTypeNode: import("./typeNodes").OptionTypeNode;
    publicKeyTypeNode: import("./typeNodes").PublicKeyTypeNode;
    setTypeNode: import("./typeNodes").SetTypeNode;
    solAmountTypeNode: import("./typeNodes").SolAmountTypeNode;
    stringTypeNode: import("./typeNodes").StringTypeNode;
    structTypeNode: import("./typeNodes").StructTypeNode;
    tupleTypeNode: import("./typeNodes").TupleTypeNode;
    fixedSizeNode: import("./sizeNodes").FixedSizeNode;
    remainderSizeNode: import("./sizeNodes").RemainderSizeNode;
    prefixedSizeNode: import("./sizeNodes").PrefixedSizeNode;
    constantPdaSeedNode: import("./pdaSeedNodes").ConstantPdaSeedNode;
    programIdPdaSeedNode: import("./pdaSeedNodes").ProgramIdPdaSeedNode;
    variablePdaSeedNode: import("./pdaSeedNodes").VariablePdaSeedNode;
    programLinkNode: import("./linkNodes").ProgramLinkNode;
    pdaLinkNode: import("./linkNodes").PdaLinkNode;
    accountLinkNode: import("./linkNodes").AccountLinkNode;
    definedTypeLinkNode: import("./linkNodes").DefinedTypeLinkNode;
    byteDiscriminatorNode: import("./discriminatorNodes").ByteDiscriminatorNode;
    fieldDiscriminatorNode: import("./discriminatorNodes").FieldDiscriminatorNode;
    sizeDiscriminatorNode: import("./discriminatorNodes").SizeDiscriminatorNode;
    pdaSeedValueNode: import("./contextualValueNodes").PdaSeedValueNode;
    accountBumpValueNode: import("./contextualValueNodes").AccountBumpValueNode;
    accountValueNode: import("./contextualValueNodes").AccountValueNode;
    argumentValueNode: import("./contextualValueNodes").ArgumentValueNode;
    conditionalValueNode: import("./contextualValueNodes").ConditionalValueNode;
    identityValueNode: import("./contextualValueNodes").IdentityValueNode;
    payerValueNode: import("./contextualValueNodes").PayerValueNode;
    pdaValueNode: import("./contextualValueNodes").PdaValueNode;
    programIdValueNode: import("./contextualValueNodes").ProgramIdValueNode;
    resolverValueNode: import("./contextualValueNodes").ResolverValueNode;
    rootNode: RootNode;
    programNode: ProgramNode;
    pdaNode: PdaNode;
    accountNode: AccountNode;
    instructionAccountNode: InstructionAccountNode;
    instructionArgumentNode: InstructionArgumentNode;
    instructionByteDeltaNode: InstructionByteDeltaNode;
    instructionNode: InstructionNode;
    instructionRemainingAccountsNode: InstructionRemainingAccountsNode;
    errorNode: ErrorNode;
    definedTypeNode: DefinedTypeNode;
};
export declare const REGISTERED_NODE_KINDS: ("arrayValueNode" | "booleanValueNode" | "enumValueNode" | "mapValueNode" | "noneValueNode" | "numberValueNode" | "setValueNode" | "someValueNode" | "structValueNode" | "tupleValueNode" | "publicKeyValueNode" | "stringValueNode" | "mapEntryValueNode" | "structFieldValueNode" | "structFieldTypeNode" | "enumEmptyVariantTypeNode" | "enumStructVariantTypeNode" | "enumTupleVariantTypeNode" | "amountTypeNode" | "arrayTypeNode" | "booleanTypeNode" | "bytesTypeNode" | "dateTimeTypeNode" | "enumTypeNode" | "mapTypeNode" | "numberTypeNode" | "optionTypeNode" | "publicKeyTypeNode" | "setTypeNode" | "solAmountTypeNode" | "stringTypeNode" | "structTypeNode" | "tupleTypeNode" | "fixedSizeNode" | "remainderSizeNode" | "prefixedSizeNode" | "constantPdaSeedNode" | "programIdPdaSeedNode" | "variablePdaSeedNode" | "programLinkNode" | "pdaLinkNode" | "accountLinkNode" | "definedTypeLinkNode" | "byteDiscriminatorNode" | "fieldDiscriminatorNode" | "sizeDiscriminatorNode" | "pdaSeedValueNode" | "accountBumpValueNode" | "accountValueNode" | "argumentValueNode" | "conditionalValueNode" | "identityValueNode" | "payerValueNode" | "pdaValueNode" | "programIdValueNode" | "resolverValueNode" | "rootNode" | "programNode" | "pdaNode" | "accountNode" | "instructionAccountNode" | "instructionArgumentNode" | "instructionByteDeltaNode" | "instructionNode" | "instructionRemainingAccountsNode" | "errorNode" | "definedTypeNode")[];
export type NodeDictionary = typeof REGISTERED_NODES;
export type NodeKind = keyof NodeDictionary;
export type Node = NodeDictionary[NodeKind];
export declare function isNode<TKind extends NodeKind>(node: Node | null | undefined, kind: TKind | TKind[]): node is NodeDictionary[TKind];
export declare function assertIsNode<TKind extends NodeKind>(node: Node | null | undefined, kind: TKind | TKind[]): asserts node is NodeDictionary[TKind];
export declare function isNodeFilter<TKind extends NodeKind>(kind: TKind | TKind[]): (node: Node | null | undefined) => node is NodeDictionary[TKind];
export declare function assertIsNodeFilter<TKind extends NodeKind>(kind: TKind | TKind[]): (node: Node | null | undefined) => node is NodeDictionary[TKind];
export declare function removeNullAndAssertIsNodeFilter<TKind extends NodeKind>(kind: TKind | TKind[]): (node: Node | null | undefined) => node is NodeDictionary[TKind];
export {};
