import { Node } from '../nodes';
import type { NodeStack } from './NodeStack';
export type NodeSelector = NodeSelectorPath | NodeSelectorFunction;
/**
 * A string that can be used to select a node in a Kinobi tree.
 * - `*` matches any node.
 * - `someText` matches the name of a node, if any.
 * - `[someNode]` matches a node of the given kind.
 * - `[someNode|someOtherNode]` matches a node with any of the given kind.
 * - `[someNode]someText` matches both the kind and the name of a node.
 * - `a.b.c` matches a node `c` such that its parent stack contains `a` and `b` in order (but not necessarily subsequent).
 */
export type NodeSelectorPath = string;
export type NodeSelectorFunction = (node: Node, stack: NodeStack) => boolean;
export declare const getNodeSelectorFunction: (selector: NodeSelector) => NodeSelectorFunction;
export declare const getConjunctiveNodeSelectorFunction: (selector: NodeSelector | NodeSelector[]) => NodeSelectorFunction;
